'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System.Collections.Generic
Imports System.IO
Imports System.Xml.XPath

Imports ICSharpCode.SharpZipLib.Zip

Namespace DotNetNuke.Modules.Reports.Installer

    <Serializable()> _
    Public Class InstallerException
        Inherits Exception

        Private _errorMessageKey As String
        Private _argument As String

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the localization key for the error message
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	01/17/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property ErrorMessageKey() As String
            Get
                Return _errorMessageKey
            End Get
            Set(ByVal value As String)
                _errorMessageKey = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the argument associated with this error
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	01/17/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property Argument() As String
            Get
                Return _argument
            End Get
            Set(ByVal value As String)
                _argument = value
            End Set
        End Property

        Public Sub New()
            MyBase.New()
        End Sub

        Public Sub New(ByVal message As String, ByVal errorMessageKey As String)
            MyBase.New(message)
            Me.ErrorMessageKey = errorMessageKey
        End Sub

        Public Sub New(ByVal message As String, ByVal errorMessageKey As String, ByVal argument As String)
            MyBase.New(message)
            Me.Argument = argument
            Me.ErrorMessageKey = errorMessageKey
        End Sub

        Public Sub New(ByVal message As String, ByVal innerException As Exception)
            MyBase.New(message, innerException)
        End Sub

        Public Sub New(ByVal message As String, ByVal innerException As Exception, ByVal argument As String)
            MyBase.New(message, innerException)
            Me.Argument = Argument
        End Sub

        Public Sub New(ByVal message As String, ByVal errorMessageKey As String, ByVal innerException As Exception)
            MyBase.New(message, innerException)
            Me.ErrorMessageKey = errorMessageKey
        End Sub

        Public Sub New(ByVal message As String, ByVal errorMessageKey As String, ByVal argument As String, ByVal innerException As Exception)
            MyBase.New(message, innerException)
            Me.Argument = argument
            Me.ErrorMessageKey = errorMessageKey
        End Sub

        Public Sub New(ByVal info As System.Runtime.Serialization.SerializationInfo, ByVal context As System.Runtime.Serialization.StreamingContext)
            MyBase.New(info, context)
            Me.Argument = info.GetString("Argument")
            Me.ErrorMessageKey = info.GetString("ErrorMessageKey")
        End Sub

        Public Overrides Sub GetObjectData(ByVal info As System.Runtime.Serialization.SerializationInfo, ByVal context As System.Runtime.Serialization.StreamingContext)
            MyBase.GetObjectData(info, context)
            info.AddValue("File", Me.Argument)
            info.AddValue("ErrorMessageKey", Me.ErrorMessageKey)
        End Sub

    End Class

    Public Class PackageInstaller

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Installs the specified package into the specified base Reports Module
        ''' folder
        ''' </summary>
        ''' <exception cref="DotNetNuke.Modules.Reports.Installer.InstallerException">There was an error during installation</exception>
        ''' <param name="BasePath">
        ''' The base path in which to install the package (usually: "/DesktopModules/Reports")
        ''' </param>
        ''' <history>
        ''' 	[anurse]	01/17/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub InstallPackage(ByVal PackageStream As Stream, ByVal BasePath As String)
            Dim packageFolder As String = Null.NullString
            Dim manifest As PackageManifest = Nothing
            Dim binFolder As String = Null.NullString
            Dim tempFile As String = Null.NullString
            Dim manifestData As Byte() = Nothing
            Dim packageZip As ZipFile = Nothing
            Try
                If HttpContext.Current Is Nothing Then
                    Throw New InstallerException("Cannot install package, no current HttpContext", "NoHttpContext.ErrorMessage")
                End If

                ' Ensure the Temp folder exists
                Dim tempFolder As String = Path.Combine( _
                        HttpContext.Current.Server.MapPath(BasePath), _
                        "InstallTemp")
                If Not Directory.Exists(tempFolder) Then
                    Directory.CreateDirectory(tempFolder)
                End If

                ' Write the Zip file to a temporary file
                tempFile = String.Format("{0}.pkg.zip", Path.Combine( _
                    tempFolder, Path.GetRandomFileName()))
                WriteStream(PackageStream, tempFile)

                ' Open the zip file from the temporary file
                packageZip = New ZipFile(tempFile)

                ' Find the "Manifest.xml" file
                Dim manifestEntry As ZipEntry = packageZip.GetEntry("Manifest.xml")
                If manifestEntry Is Nothing Then
                    Throw New InstallerException("Package does not have a manifest file", "MissingManifest.ErrorMessage")
                End If

                ' Copy the manifest to a memory buffer
                Dim manifestStream As Stream = packageZip.GetInputStream(manifestEntry)
                Dim manifestBuffer As New MemoryStream(manifestStream.Length)
                StreamToStream(manifestStream, manifestBuffer)
                manifestData = manifestBuffer.ToArray()
                manifestBuffer.Close()

                
                ' Open the manifest file as an XPathDocuement and parse it
                manifestBuffer = New MemoryStream(manifestData)
                Dim manifestDoc As New XPathDocument(manifestBuffer)
                manifestBuffer.Close()

                ' Gross... but XPathDocument.ctor(Stream) closes our stream and
                ' we still need it :(
                manifestBuffer = New MemoryStream(manifestData)

                manifest = New PackageManifest(manifestDoc)

                ' Install the assemblies into the /bin folder
                binFolder = HttpContext.Current.Server.MapPath("~/bin")
                For Each asm As String In manifest.Assemblies
                    ' Get the assembly's zip entry
                    Dim asmEntry As ZipEntry = packageZip.GetEntry(asm)
                    If asmEntry Is Nothing Then
                        Throw New InstallerException("Assembly referenced in manifest not found in package", "MissingAssembly.ErrorMessage", asm)
                    End If

                    ' Get the file name for the new file
                    Dim fileName As String = GetFileName(asmEntry, binFolder, True)

                    ' Write the zip entry to the new file
                    WriteZipEntry(packageZip, asmEntry, fileName)
                Next

                ' Create a folder for the files
                packageFolder = GetPackageFolder(BasePath, manifest.Name, manifest.Type)
                If Not Directory.Exists(packageFolder) Then
                    Directory.CreateDirectory(packageFolder)
                End If

                ' Write the manifest file 

                WriteStream(manifestBuffer, Path.Combine(packageFolder, "Manifest.xml.config"))
                manifestBuffer.Close()

                ' Iterate over the files and unpack them
                For Each file As FileManifest In manifest.Files
                    ' Get the file's zip entry
                    Dim fileEntry As ZipEntry = packageZip.GetEntry(file.Source)
                    If fileEntry Is Nothing Then
                        Throw New InstallerException("File referenced in manifest not found in package", "MissingFile.ErrorMessage", file.Source)
                    End If

                    ' Get the file name for the new file
                    Dim fileName As String = GetFileName(fileEntry, packageFolder, False)

                    ' Write the zip entry to the new file
                    WriteZipEntry(packageZip, fileEntry, fileName)
                Next
            Catch ex As Exception
                If manifest IsNot Nothing Then
                    If Not String.IsNullOrEmpty(binFolder) Then
                        RemoveAssemblies(binFolder, manifest.Assemblies)
                    End If

                    If Not String.IsNullOrEmpty(packageFolder) Then
                        RemoveFiles(packageFolder, manifest.Files)
                        If Directory.Exists(packageFolder) Then
                            Directory.Delete(packageFolder, True)
                        End If
                    End If
                End If

                If Not TypeOf ex Is InstallerException Then
                    Throw New InstallerException("Unknown error in installation", "UnknownError.ErrorMessage", ex)
                Else
                    Throw
                End If
            Finally
                If packageZip IsNot Nothing Then
                    packageZip.Close()
                End If
                If Not String.IsNullOrEmpty(tempFile) AndAlso File.Exists(tempFile) Then
                    File.Delete(tempFile)
                End If
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Uninstalls the package with the specified name and type
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	02/05/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub UninstallPackage(ByVal BasePath As String, ByVal Manifest As PackageManifest)
            Try
                If HttpContext.Current Is Nothing Then
                    Throw New InstallerException("Cannot uninstall package, no current HttpContext", "NoHttpContext.ErrorMessage")
                End If

                Dim packageFolder As String = GetPackageFolder(BasePath, Manifest.Name, Manifest.Type)
                Dim binFolder As String = HttpContext.Current.Server.MapPath("~/bin")

                ' Delete the package assemblies
                For Each asm As String In Manifest.Assemblies
                    Dim filePath As String = Path.Combine(binFolder, asm)
                    If File.Exists(filePath) Then File.Delete(filePath)
                Next

                ' Delete the package directory
                If Directory.Exists(packageFolder) Then
                    Directory.Delete(packageFolder, True)
                End If
            Catch ex As Exception
                Throw New InstallerException("An error occurred during uninstallation, some files or folders may not have been removed and must be removed manually", "UninstallError.ErrorMessage", ex)
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets a list of the extensions installed in the specified base folder
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	02/05/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetExtensions(ByVal BaseFolder As String) As List(Of PackageManifest)
            Return GetExtensions(BaseFolder, True)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets a list of the extensions installed in the specified base folder
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	02/05/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetExtensions(ByVal BaseFolder As String, ByVal IncludeLegacyPackages As Boolean) As List(Of PackageManifest)
            Dim baseDir As New DirectoryInfo(BaseFolder)
            Dim manifests As New List(Of PackageManifest)()

            GetExtensions(baseDir, manifests, "Visualizers", IncludeLegacyPackages)
            GetExtensions(baseDir, manifests, "DataSources", IncludeLegacyPackages)

            Return manifests
        End Function

        Private Shared Sub RemoveFiles(ByVal PackageFolder As String, ByVal Files As List(Of FileManifest))
            For Each file As FileManifest In Files
                Dim filePath As String = Path.Combine(PackageFolder, file.Destination)
                If IO.File.Exists(filePath) Then
                    IO.File.Delete(filePath)
                End If
            Next
        End Sub

        Private Shared Sub RemoveAssemblies(ByVal BinFolder As String, ByVal Assemblies As List(Of String))
            For Each asm As String In Assemblies
                Dim asmPath As String = Path.Combine(BinFolder, asm)
                If IO.File.Exists(asmPath) Then
                    IO.File.Delete(asmPath)
                End If
            Next
        End Sub

        Private Shared Sub GetExtensions(ByVal baseDir As DirectoryInfo, ByVal manifests As List(Of PackageManifest), ByVal folderName As String, ByVal IncludeLegacyPackages As Boolean)
            Dim dirs As DirectoryInfo() = baseDir.GetDirectories(folderName)
            If (dirs.Length > 0) AndAlso (dirs(0) IsNot Nothing) Then
                For Each dir As DirectoryInfo In dirs(0).GetDirectories()
                    Dim manifestFiles As FileInfo() = dir.GetFiles("Manifest.xml.config")
                    If manifestFiles.Length < 1 Then
                        If IncludeLegacyPackages Then
                            manifests.Add(BuildManifest(dir, GetPackageType(dirs(0).Name)))
                        End If
                        Continue For
                    End If
                    Dim xpdoc As New XPathDocument(manifestFiles(0).OpenRead())
                    Dim manifest As PackageManifest = Nothing
                    Try
                        manifest = New PackageManifest(xpdoc)
                    Catch ex As InstallerException
                        manifest = Nothing
                    End Try
                    If manifest IsNot Nothing Then manifests.Add(manifest)
                Next
            End If
        End Sub

        Private Shared Function BuildManifest(ByVal PackageDir As DirectoryInfo, ByVal Type As PackageType) As PackageManifest
            Dim manifest As New PackageManifest(PackageDir.Name, Type)
            manifest.ModuleVersion = New Version(4, 4, 2)
            manifest.PackageVersion = New Version(1, 0)
            manifest.Description = String.Empty
            manifest.Vendor = String.Empty
            Return manifest
        End Function

        Private Shared Function GetPackageFolder(ByVal BasePath As String, ByVal Name As String, ByVal Type As PackageType) As String
            Dim rootFolder As String = GetRootFolder(Type)
            Return HttpContext.Current.Server.MapPath( _
                String.Format("{0}/{1}/{2}", BasePath, rootFolder, Name))
        End Function

        Private Shared Function GetRootFolder(ByVal Type As PackageType) As String
            Select Case Type
                Case PackageType.DataSource
                    Return "DataSources"
                Case PackageType.Visualizer
                    Return "Visualizers"
                Case Else
                    Throw New InstallerException("Unknown Package Type encountered", "UnknownPackageType.Text", Type.ToString())
            End Select
        End Function

        Private Shared Function GetPackageType(ByVal FolderName As String) As PackageType
            Select Case FolderName
                Case "DataSources"
                    Return PackageType.DataSource
                Case "Visualizers"
                    Return PackageType.Visualizer
                Case Else
                    Throw New InstallerException("Unknown Package Type encountered", "UnknownPackageType.Text", FolderName.ToString())
            End Select
        End Function


        Private Shared Function GetFileName(ByVal Entry As ZipEntry, ByVal BaseFolder As String, ByVal StripFolders As Boolean) As String
            ' Get the actual file name (not the path)
            Dim fileName As String = Entry.Name
            If StripFolders Then
                fileName = Path.GetFileName(fileName)
            End If
            fileName = fileName.Replace("/"c, Path.DirectorySeparatorChar)

            ' Append the bin folder path and return
            Return Path.Combine(BaseFolder, fileName)
        End Function

        Private Shared Sub StreamToStream(ByVal SourceStream As Stream, ByVal DestStream As Stream)
            Dim buf(1024) As Byte
            Dim count As Integer = 0
            Do
                ' Read the chunk from the source
                count = SourceStream.Read(buf, 0, 1024)

                ' Write the chunk to the destination
                DestStream.Write(buf, 0, count)
            Loop While count > 0
            DestStream.Flush()
        End Sub

        Private Shared Sub WriteStream(ByVal Stream As Stream, ByVal TargetFileName As String)
            ' Create the file
            Dim file As New FileInfo(TargetFileName)
            If file.Exists Then
                ' Delete it
                file.Delete()
                'Return ' Silent failure
                ''Throw New InstallerException("File to be installed already exists", "FileExists.ErrorMessage", TargetFileName)
            End If
            If Not file.Directory.Exists Then
                file.Directory.Create()
            End If

            Dim fileStrm As Stream = file.Create()

            StreamToStream(Stream, fileStrm)

            ' Close the stream
            fileStrm.Close()
        End Sub

        Private Shared Sub WriteZipEntry(ByVal Zip As ZipFile, ByVal Entry As ZipEntry, ByVal TargetFileName As String)
            Try
                ' Create the file and get the zip stream
                Dim zipStrm As Stream = Zip.GetInputStream(Entry)

                WriteStream(zipStrm, TargetFileName)
            Catch ex As Exception
                Throw New InstallerException("Error unpacking package file", "ErrorUnpacking.ErrorMessage", Entry.Name, ex)
            End Try
        End Sub

    End Class

End Namespace
