'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System.Collections.Generic
Imports System.Xml.XPath

Namespace DotNetNuke.Modules.Reports.Installer

    Public Enum PackageType
        Visualizer
        DataSource ' Reserved for future use (version 5.0)
    End Enum

    Public Class PackageManifest
        Private _moduleVersion As Version
        Private _packageVersion As Version
        Private _fileName As String
        Private _vendor As String
        Private _name As String
        Private _type As PackageType
        Private _description As String
        Private _assemblies As IList(Of String) = New List(Of String)
        Private _files As IList(Of FileManifest) = New List(Of FileManifest)

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the version of the Reports Module for which this extension
        ''' package was designed
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	01/15/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property ModuleVersion() As Version
            Get
                Return _moduleVersion
            End Get
            Set(ByVal value As Version)
                _moduleVersion = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the version of the package represented in this manifest
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	02/06/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property PackageVersion() As Version
            Get
                Return _packageVersion
            End Get
            Set(ByVal value As Version)
                _packageVersion = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the name of the archive file that contained this manifest
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	01/15/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property FileName() As String
            Get
                Return _fileName
            End Get
            Set(ByVal value As String)
                _fileName = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the vendor of the package
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	02/05/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property Vendor() As String
            Get
                Return _vendor
            End Get
            Set(ByVal value As String)
                _vendor = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the name of the package represented in this manifest
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	01/15/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property Name() As String
            Get
                Return _name
            End Get
            Set(ByVal value As String)
                _name = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets a short description of the package represented in this manifest
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	01/15/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property Description() As String
            Get
                Return _description
            End Get
            Set(ByVal value As String)
                _description = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the type of the package represented in this manifest
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	01/15/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property Type() As PackageType
            Get
                Return _type
            End Get
            Set(ByVal value As PackageType)
                _type = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets a collection of paths to assemblies to be installed from the package archive
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	01/15/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Assemblies() As IList(Of String)
            Get
                Return _assemblies
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets a collection of file manifests representing files to be installed 
        ''' from the package archive
        ''' </summary>
        ''' <history>
        ''' 	[anurse]	01/15/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Files() As IList(Of FileManifest)
            Get
                Return _files
            End Get
        End Property

        Public Sub New(ByVal name As String, ByVal type As PackageType)
            Me.Name = name
            Me.Type = type
        End Sub

        Public Sub New(ByVal manifestDocument As XPathDocument)
            ' Find the Package root element
            Dim manifestNav As XPathNavigator = manifestDocument.CreateNavigator()
            Dim rootNav As XPathNavigator = manifestNav.SelectSingleNode("Package")

            ' Extract the attributes
            Me.Type = System.Enum.Parse(GetType(PackageType), rootNav.GetAttribute("type", String.Empty))
            Me.Name = rootNav.GetAttribute("name", String.Empty)
            Me.Vendor = rootNav.GetAttribute("vendor", String.Empty)
            Me.ModuleVersion = New Version(rootNav.GetAttribute("moduleVersion", String.Empty))
            Me.PackageVersion = New Version(rootNav.GetAttribute("version", String.Empty))
            If (Me.ModuleVersion.Major <> 4) OrElse (Me.ModuleVersion.Minor <> 5) Then
                Throw New InstallerException("Package is not supported on this version of the reports module", "VersionMismatch.ErrorMessage", Me.ModuleVersion.ToString())
            End If

            ' Extract the value of the description element
            Dim descNav As XPathNavigator = rootNav.SelectSingleNode("Description")
            Me.Description = descNav.InnerXml

            ' Find the Assemblies element and read its children
            Dim asmsNav As XPathNavigator = rootNav.SelectSingleNode("Assemblies")
            If asmsNav IsNot Nothing Then
                Dim asmNodes As XPathNodeIterator = asmsNav.Select("Assembly")
                If asmNodes IsNot Nothing Then
                    For Each asmNav As XPathNavigator In asmNodes
                        Me.Assemblies.Add(asmNav.InnerXml)
                    Next
                End If
            End If

            ' Find the Files element and read its children
            Dim filesNav As XPathNavigator = rootNav.SelectSingleNode("Files")
            If filesNav IsNot Nothing Then
                Dim fileNodes As XPathNodeIterator = filesNav.Select("File")
                If fileNodes IsNot Nothing Then
                    For Each fileNav As XPathNavigator In fileNodes
                        Dim dest As String = fileNav.GetAttribute("dest", String.Empty)
                        If String.IsNullOrEmpty(dest) Then
                            dest = fileNav.InnerXml
                        End If
                        Me.Files.Add(New FileManifest(fileNav.InnerXml, dest))
                    Next
                End If
            End If
        End Sub
    End Class

    Public Class FileManifest
        Private _source As String
        Private _destination As String

        Public Property Source() As String
            Get
                Return _source
            End Get
            Set(ByVal value As String)
                _source = value
            End Set
        End Property

        Public Property Destination() As String
            Get
                Return _destination
            End Get
            Set(ByVal value As String)
                _destination = value
            End Set
        End Property

        Public Sub New(ByVal source As String, ByVal destination As String)
            Me.Source = source
            Me.Destination = destination
        End Sub

    End Class

End Namespace
